@@ -19,20 +19,22 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#  include "config.h"
 #endif
 
+#include <string.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <fcntl.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
-#include <libgnome/gnome-help.h>
+#include <libgnomeui/gnome-help.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <glade/glade.h>
 
 #ifdef USE_HAL
-#  include <hal/libhal.h>
+# include <libhal.h>
 #endif
 
 #include "gfloppy.h"
@@ -153,6 +155,37 @@ start_format (void)
 
 #define HAL_FLOPPY_DEV_KEY "block.device"
 
+static LibHalContext *
+get_hal_context ()
+{
+	LibHalContext *hal_ctx;
+	hal_ctx = NULL;
+
+	hal_ctx = libhal_ctx_new ();
+	if (hal_ctx) {
+		DBusConnection *dbus_conn;
+		DBusError error;
+
+		dbus_error_init (&error);
+		dbus_conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
+
+		if (dbus_error_is_set (&error)) {
+			dbus_error_free (&error);
+			return NULL;
+		}
+
+		libhal_ctx_set_dbus_connection (hal_ctx, dbus_conn);
+
+		if (!libhal_ctx_init (hal_ctx, &error)) {
+			dbus_error_free (&error);
+			libhal_ctx_free (hal_ctx);
+ 			return NULL;
+ 		}
+	}
+
+	return hal_ctx;
+}
+
 static FloppyStatus *
 get_hal_floppies (gint *num_devices, gint *num_ok, LibHalContext *ctx)
 {
@@ -160,10 +193,11 @@ get_hal_floppies (gint *num_devices, gint *num_ok, LibHalContext *ctx)
 	gchar           **device_list;
 	FloppyStatus *floppies;
 
-	device_list = hal_manager_find_device_string_match (ctx,
-							    HAL_FLOPPY_KEY, 
-							    HAL_FLOPPY_VALUE, 
-							    num_devices); 
+	device_list = libhal_manager_find_device_string_match (ctx,
+							       HAL_FLOPPY_KEY, 
+							       HAL_FLOPPY_VALUE, 
+							       num_devices, 
+							       NULL); 
 
 	floppies = NULL;
 	
@@ -174,9 +208,10 @@ get_hal_floppies (gint *num_devices, gint *num_ok, LibHalContext *ctx)
 
 		for (i = 0; i < *num_devices; i++) {
 			floppies[i].device_path = 
-				hal_device_get_property_string (ctx, 
+				libhal_device_get_property_string (ctx, 
 								device_list[i], 
-								HAL_FLOPPY_DEV_KEY);
+								HAL_FLOPPY_DEV_KEY,
+								NULL);
 
 			floppies[i].status = test_floppy_device (floppies[i].device_path);
 
@@ -184,7 +219,7 @@ get_hal_floppies (gint *num_devices, gint *num_ok, LibHalContext *ctx)
 				(*num_ok)++;
 		}
 		
-		hal_free_string_array (device_list);
+		libhal_free_string_array (device_list);
 	} else {
 		*num_devices = 1;
 		
@@ -300,7 +335,6 @@ init_devices (gpointer ctx)
 
 		if (ok_devices_present == 0) {
 			GtkWidget *toplevel;
-			GtkWidget *dialog;
 
 			/* FIXME: Let's assume that there's only a problem with /dev/fd0 */
 			switch (floppies[0].status) {
@@ -518,6 +552,8 @@ on_format_button_clicked (GtkWidget *widget, gpointer user_data)
 int
 main (int argc, char *argv[])
 {
+	GnomeProgram *program;
+	GOptionContext *context;
 	GConfClient *client;
 	GFloppyConfig config;
 	GtkWidget *fs_entry;
@@ -527,33 +563,36 @@ main (int argc, char *argv[])
 	GtkWidget *label;
 	GtkSizeGroup *size_group;
 	GSList *l;
-        gint base_size;
-
+	gchar *label_text;
 
 	/* This is for the libhal context, should we need it */
 	gpointer ctx;
 
-	struct poptOption gfloppy_opts[] = {
-		{"device", '\0', POPT_ARG_STRING, NULL, 0, NULL, NULL},
-		{NULL, '\0', 0, NULL, 0, NULL, NULL}
+	static const GOptionEntry options[] =  {
+	  { "device", 0, 0, G_OPTION_ARG_STRING, &(floppy.device),
+	    N_("The device to format"), N_("DEVICE") },
+	  { NULL }
 	};
 
 	floppy.device = NULL;
-	gfloppy_opts[0].arg = &(floppy.device);
-	gfloppy_opts[0].descrip = _("The device to format");
-	gfloppy_opts[0].argDescrip = _("DEVICE");
 
 	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
-	gnome_program_init ("gfloppy",VERSION, LIBGNOMEUI_MODULE,
-			    argc, argv, GNOME_PARAM_POPT_TABLE, gfloppy_opts,
-                            GNOME_PARAM_POPT_FLAGS, 0,
-                            GNOME_PARAM_APP_DATADIR,DATADIR,NULL);
+	context = g_option_context_new (_("- Floppy Formatter"));
+	
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
+	program = gnome_program_init
+	  (argv[0], VERSION, LIBGNOMEUI_MODULE,
+	   argc, argv,
+	   GNOME_PARAM_GOPTION_CONTEXT, context,
+	   GNOME_PARAM_APP_DATADIR,DATADIR,
+	   GNOME_PARAM_NONE);
 
 #ifdef USE_HAL
-	ctx = (gpointer) hal_initialize (NULL, FALSE);
+	ctx = (gpointer)get_hal_context ();
 #endif
 
 	gtk_window_set_default_icon_name ("gnome-dev-floppy");
@@ -567,8 +606,6 @@ main (int argc, char *argv[])
 	if (xml == NULL)
 		g_error ("Cannot load/find gfloppy2.glade");
 
-	init_devices (ctx);
-
 	/* We get the different widgets */
 	
 	toplevel = glade_xml_get_widget (xml, "toplevel");
@@ -628,6 +665,22 @@ main (int argc, char *argv[])
 	default_mode_rb = (GtkWidget *) g_slist_nth (l, GFLOPPY_FORMAT_THOROUGH - config.default_formatting_mode)->data;
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (default_mode_rb), TRUE);
 
+	/* bold header labels per HIG; done here to avoid markup in translations */
+	label = glade_xml_get_widget (xml, "physical_label");
+	label_text = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", gtk_label_get_text (GTK_LABEL (label)));
+	gtk_label_set_markup (GTK_LABEL (label), label_text);
+	g_free (label_text);
+
+	label = glade_xml_get_widget (xml, "filesystem_label");
+	label_text = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", gtk_label_get_text (GTK_LABEL (label)));
+	gtk_label_set_markup (GTK_LABEL (label), label_text);
+	g_free (label_text);
+
+	label = glade_xml_get_widget (xml, "formatting_label");
+	label_text = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", gtk_label_get_text (GTK_LABEL (label)));
+	gtk_label_set_markup (GTK_LABEL (label), label_text);
+	g_free (label_text);
+
 	/* check the UI consistency for a couple of settings */
 	g_signal_emit_by_name (type_option, "changed", NULL);
 
@@ -655,6 +708,8 @@ main (int argc, char *argv[])
 		gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET(fs_entry));
 	}
 
+	init_devices (ctx);
+
 	gtk_widget_show (GTK_WIDGET (toplevel));
 
 	gtk_main ();
@@ -677,9 +732,17 @@ main (int argc, char *argv[])
 	g_object_unref (G_OBJECT (client));
 
 #ifdef USE_HAL
-	hal_shutdown (ctx);
+	if (ctx != NULL) {
+		DBusError derror;
+		libhal_ctx_shutdown ((LibHalContext *)ctx, &derror);
+		if (dbus_error_is_set (&derror))
+			dbus_error_free (&derror);
+
+		libhal_ctx_free ((LibHalContext *)ctx);
+	}
 #endif
 
+	g_object_unref (program);
 	return 0;
 }
 
